"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRegistryCredVarsFromPackageFile = exports.extractHeaderCommand = exports.allowedPipOptions = exports.optionsWithArguments = exports.disallowedPipOptions = exports.constraintLineRegex = exports.getExecOptions = exports.getPipToolsConstraint = exports.getPythonConstraint = void 0;
const tslib_1 = require("tslib");
const is_1 = tslib_1.__importDefault(require("@sindresorhus/is"));
const shlex_1 = require("shlex");
const upath_1 = tslib_1.__importDefault(require("upath"));
const logger_1 = require("../../../logger");
const array_1 = require("../../../util/array");
const fs_1 = require("../../../util/fs");
const util_1 = require("../../../util/fs/util");
const hostRules = tslib_1.__importStar(require("../../../util/host-rules"));
const regex_1 = require("../../../util/regex");
function getPythonConstraint(config) {
    const { constraints = {} } = config;
    const { python } = constraints;
    if (python) {
        logger_1.logger.debug('Using python constraint from config');
        return python;
    }
    return undefined;
}
exports.getPythonConstraint = getPythonConstraint;
// TODO(not7cd): rename to getPipToolsVersionConstraint, as constraints have their meaning in pip
function getPipToolsConstraint(config) {
    const { constraints = {} } = config;
    const { pipTools } = constraints;
    if (is_1.default.string(pipTools)) {
        logger_1.logger.debug('Using pipTools constraint from config');
        return pipTools;
    }
    return '';
}
exports.getPipToolsConstraint = getPipToolsConstraint;
async function getExecOptions(config, cwd, extraEnv) {
    const constraint = getPythonConstraint(config);
    const pipToolsConstraint = getPipToolsConstraint(config);
    const execOptions = {
        cwd: (0, util_1.ensureLocalPath)(cwd),
        docker: {},
        userConfiguredEnv: config.env,
        toolConstraints: [
            {
                toolName: 'python',
                constraint,
            },
            {
                toolName: 'pip-tools',
                constraint: pipToolsConstraint,
            },
        ],
        extraEnv: {
            PIP_CACHE_DIR: await (0, fs_1.ensureCacheDir)('pip'),
            PIP_NO_INPUT: 'true', // ensure pip doesn't block forever waiting for credentials on stdin
            PIP_KEYRING_PROVIDER: 'import',
            PYTHON_KEYRING_BACKEND: 'keyrings.envvars.keyring.EnvvarsKeyring',
            ...extraEnv,
        },
    };
    return execOptions;
}
exports.getExecOptions = getExecOptions;
exports.constraintLineRegex = (0, regex_1.regEx)(/^(#.*?\r?\n)+# {4}(?<command>\S*)(?<arguments> .*?)?\r?\n/);
exports.disallowedPipOptions = [
    '--no-header', // header is required by this manager
];
exports.optionsWithArguments = [
    '--output-file',
    '--extra',
    '--extra-index-url',
    '--resolver',
    '--constraint',
];
exports.allowedPipOptions = [
    '-v',
    '--all-extras',
    '--allow-unsafe',
    '--generate-hashes',
    '--no-emit-index-url',
    '--emit-index-url',
    '--strip-extras',
    '--index-url',
    ...exports.optionsWithArguments,
];
// TODO(not7cd): test on all correct headers, even with CUSTOM_COMPILE_COMMAND
function extractHeaderCommand(content, fileName) {
    const compileCommand = exports.constraintLineRegex.exec(content);
    if (compileCommand?.groups === undefined) {
        throw new Error(`Failed to extract command from header in ${fileName} ${content}`);
    }
    logger_1.logger.trace(`pip-compile: found header in ${fileName}: \n${compileCommand[0]}`);
    const command = compileCommand.groups.command;
    const argv = [command];
    const isCustomCommand = command !== 'pip-compile';
    if (compileCommand.groups.arguments) {
        argv.push(...(0, shlex_1.split)(compileCommand.groups.arguments));
    }
    logger_1.logger.debug({ fileName, argv, isCustomCommand }, `pip-compile: extracted command from header`);
    const result = {
        argv,
        command,
        isCustomCommand,
        outputFile: '',
        sourceFiles: [],
    };
    for (const arg of argv.slice(1)) {
        // TODO(not7cd): check for "--option -- argument" case
        if (!arg.startsWith('-')) {
            result.sourceFiles.push(arg);
            continue;
        }
        throwForDisallowedOption(arg);
        throwForNoEqualSignInOptionWithArgument(arg);
        throwForUnknownOption(arg);
        if (arg.includes('=')) {
            const [option, value] = arg.split('=');
            if (option === '--extra') {
                result.extra = result.extra ?? [];
                result.extra.push(value);
            }
            else if (option === '--extra-index-url') {
                result.extraIndexUrl = result.extraIndexUrl ?? [];
                result.extraIndexUrl.push(value);
                // TODO: add to secrets? next PR
            }
            else if (option === '--constraint') {
                result.constraintsFiles = result.constraintsFiles ?? [];
                result.constraintsFiles.push(value);
            }
            else if (option === '--output-file') {
                if (result.outputFile) {
                    throw new Error('Cannot use multiple --output-file options');
                }
                result.outputFile = upath_1.default.normalize(value);
            }
            else if (option === '--index-url') {
                if (result.indexUrl) {
                    throw new Error('Cannot use multiple --index-url options');
                }
                result.indexUrl = value;
                // TODO: add to secrets? next PR
            }
            else {
                logger_1.logger.warn(`pip-compile: option ${arg} not handled`);
            }
            continue;
        }
        if (arg === '--no-emit-index-url') {
            result.noEmitIndexUrl = true;
            continue;
        }
        if (arg === '--emit-index-url') {
            result.emitIndexUrl = true;
            continue;
        }
        if (arg === '--all-extras') {
            result.allExtras = true;
            continue;
        }
        logger_1.logger.warn(`pip-compile: option ${arg} not handled`);
    }
    logger_1.logger.trace({
        ...result,
    }, 'Parsed pip-compile command from header');
    if (result.noEmitIndexUrl && result.emitIndexUrl) {
        throw new Error('Cannot use both --no-emit-index-url and --emit-index-url');
    }
    if (result.sourceFiles.length === 0) {
        throw new Error('No source files detected in command, pass at least one package file explicitly');
    }
    return result;
}
exports.extractHeaderCommand = extractHeaderCommand;
function throwForDisallowedOption(arg) {
    if (exports.disallowedPipOptions.includes(arg)) {
        throw new Error(`Option ${arg} not allowed for this manager`);
    }
}
function throwForNoEqualSignInOptionWithArgument(arg) {
    if (exports.optionsWithArguments.includes(arg)) {
        throw new Error(`Option ${arg} must have equal sign '=' separating it's argument`);
    }
}
function throwForUnknownOption(arg) {
    if (arg.includes('=')) {
        const [option] = arg.split('=');
        if (exports.allowedPipOptions.includes(option)) {
            return;
        }
    }
    if (exports.allowedPipOptions.includes(arg)) {
        return;
    }
    throw new Error(`Option ${arg} not supported (yet)`);
}
function getRegistryCredEnvVars(url, index) {
    const hostRule = hostRules.find({ url: url.href });
    logger_1.logger.debug(hostRule, `Found host rule for url ${url.href}`);
    const ret = {};
    if (!!hostRule.username || !!hostRule.password) {
        ret[`KEYRING_SERVICE_NAME_${index}`] = url.hostname;
        ret[`KEYRING_SERVICE_USERNAME_${index}`] = hostRule.username ?? '';
        ret[`KEYRING_SERVICE_PASSWORD_${index}`] = hostRule.password ?? '';
    }
    return ret;
}
function cleanUrl(url) {
    try {
        // Strip everything but protocol, host, and port
        const urlObj = new URL(url);
        return new URL(urlObj.origin);
    }
    catch {
        return null;
    }
}
function getRegistryCredVarsFromPackageFile(packageFile) {
    const urls = [
        ...(packageFile?.registryUrls ?? []),
        ...(packageFile?.additionalRegistryUrls ?? []),
    ];
    const uniqueHosts = new Set(urls.map(cleanUrl).filter(array_1.isNotNullOrUndefined));
    let allCreds = {};
    for (const [index, host] of [...uniqueHosts].entries()) {
        const hostCreds = getRegistryCredEnvVars(host, index);
        allCreds = {
            ...allCreds,
            ...hostCreds,
        };
    }
    return allCreds;
}
exports.getRegistryCredVarsFromPackageFile = getRegistryCredVarsFromPackageFile;
//# sourceMappingURL=common.js.map